/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerProxy.java 9871 2006-11-27 09:55:29 +0000 (Mon, 27 Nov 2006) danesa $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.management.monitoring;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.RuntimeOperationsException;

import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.jmx.CatalinaObjectName;
import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jmx.ManagementException;
import org.objectweb.util.monolog.api.BasicLevel;


/**
 * Implement proxy MBean representing servers in the domain on the master side.
 * Implement state transition diagram.
 * @author Adriana Danes
 */
public class ServerProxy extends JMXProxy implements ServerProxyMBean {

    private String j2eeObjectName;

    private ObjectName jvmOn = null;
    private ObjectName jtaOn = null;
    private ObjectName wpoolOn = null;

    /**
     * Name of the host as given by InetAddress.getLoaclHost().
     * (node attribute from JVM MBean)
     */
    private String hostName = null;
    private String JOnASVersion = null;
    private String javaVendor = null;
    private String javaVersion = null;
    private Long currentUsedMemory = new Long(-1);
    private Long currentTotalMemory = new Long(-1);
    private int allThreadsCount;
    private String protocols = null;
    // infos jdk5
    private String loadCPU = null;
    // Tomcat
    private boolean tomcat = false;
    private int maxThreadsByConnectorTomcat = 0;
    private int currentThreadCountByConnectorTomcat = 0;
    private int currentThreadBusyByConnectorTomcat = 0;
    private long bytesReceivedByConnectorTomcat = 0;
    private long bytesSentByConnectorTomcat = 0;
    private int errorCountByConnectorTomcat = 0;
    private long processingTimeByConnectorTomcat = 0;
    private int requestCountByConnectorTomcat = 0;
    //Tx
    private int totalBegunTransactions;
    private int totalCommittedTransactions;
    private int totalCurrentTransactions;
    private int totalExpiredTransactions;
    private int totalRolledbackTransactions;
    // Worker Pool
    private int currentWorkerPoolSize;
    private int maxWorkerPoolSize;
    private int minWorkerPoolSize;
    // JCA
    private int busyMaxJCAConnection;
    private int busyMinJCAConnection;
    private int connectionFailuresJCAConnection;
    private int connectionLeaksJCAConnection;
    private int currentBusyJCAConnection;
    private int currentInTxJCAConnection;
    private int currentOpenedJCAConnection;
    private int currentWaitersJCAConnection;
    private int initPoolSizeJCAConnection;
    private int poolMaxSizeJCAConnection;
    private int poolMinSizeJCAConnection;
    private int poolMaxWaitersJCAConnection;
    private int poolMaxWaitTimeJCAConnection;
    private int poolMaxOpentimeJCAConnection;
    private int physicalConnectionOpenedCountJCAConnection;
    private int pstmtMaxJCAConnection;
    private int rejectedFullJCAConnection;
    private int rejectedOpenJCAConnection;
    private int rejectedOtherJCAConnection;
    private int rejectedTimeoutJCAConnection;
    private int xaServedOpenJCAConnection;
    private int waiterCountJCAConnection;
    private int waitersHighJCAConnection;
    private int waitersHighRecentJCAConnection;
    private int waitingHighJCAConnection;
    private int waitingHighRecentJCAConnection;
    private int waitingTimeJCAConnection;
    // JMS
    private int jmsQueuesNbMsgsReceiveSinceCreation;
    private int jmsQueuesNbMsgsSendToDMQSinceCreation;
    private int jmsQueuesNbMsgsDeliverSinceCreation;
    private int jmsTopicsNbMsgsReceiveSinceCreation;
    private int jmsTopicsNbMsgsSendToDMQSinceCreation;
    private int jmsTopicsNbMsgsDeliverSinceCreation;
    // EJB
    private int currentNumberOfBMP;
    private int currentNumberOfCMP;
    private int currentNumberOfEJB;
    private int currentNumberOfMDB;
    private int currentNumberOfSBF;
    private int currentNumberOfSBL;
    private int currentNumberOfEBInMemory;
    private int maxNumberOfEBInMemory;
    private int minPoolSizeEB;
    private int poolSizeEB;
    private int currentNumberOfSBFInMemory;
    private int maxNumberOfSBFInMemory;
    private int minPoolSizeSBF;
    private int poolSizeSBF;
    private int currentNumberOfSBLInMemory;
    private int maxNumberOfSBLInMemory;
    private int minPoolSizeSBL;
    private int poolSizeSBL;


    /**
     * Default value for maxcheck
     */
    private static final int MAXCHECK_DEFAULT = 5;

    /**
     * Number of retries when detecting a broken connection
     */
    private int maxcheck = MAXCHECK_DEFAULT;

    /**
     * number of checks in UNREACHABLE case in the case of a server with discovery
     */
    private int nbcheck = 0;

    private boolean hasDiscovery = false;

    /**
     * Ref to the ClusterDaemon that can monitor the Server
     * null if no ClusterDaemon declared or found.
     */
    private ClusterDaemonProxy clusterdaemon = null;

    /**
     * MBeanServer
     */
    private MBeanServer mbeanServer = null;

    /**
     * Constructor used when a server is discovered
     * @param serverName the name of the managed server
     * @param jmxService the current jmx service reference
     */
    public ServerProxy(DomainMonitor dm, String serverName) {
        super(dm, serverName, null);
        logger.log(BasicLevel.DEBUG, name);
    }


    /**
     * Constructor used for a new domain.xml entry
     * @param serverName the name of the managed server
     * @param urls Collection of urls to connect to the server
     * @param cdp possible clusterdaemon
     */
    public ServerProxy(DomainMonitor dm, String serverName, Collection urls, ClusterDaemonProxy cdp) {
        super(dm, serverName, urls);
        clusterdaemon = cdp;
        logger.log(BasicLevel.DEBUG, name);
    }

    /**
     * Notification from discovery: RUNNING
     * @param urls for connection
     */
    public void notifyStarting(Collection urls) {
        logger.log(BasicLevel.DEBUG, name);
        switch (state) {
            case RUNNING:
                logger.log(BasicLevel.DEBUG, "Already running");
                break;
            case FAILED:
                if (connect(urls)) {
                    state = RUNNING;
                    getMonitoringInfo();
                }
                break;
            case UNKNOWN:
                logger.log(BasicLevel.WARN, "Running now with discovery");
            case INITIAL:
            case UNREACHABLE:
            case STOPPED:
            case STARTING:
                if (connect(urls)) {
                    state = RUNNING;
                    getMonitoringInfo();
                } else {
                    state = FAILED;
                }
                break;
        }
        hasDiscovery = true;
    }

    /**
     * Notification from discovery: STOPPING
     */
    public void notifyStopping() {
        logger.log(BasicLevel.DEBUG, name);
        switch (state) {
            case STOPPED:
                logger.log(BasicLevel.DEBUG, "Already stopped");
                break;
            case UNKNOWN:
                logger.log(BasicLevel.WARN, "Running now with discovery");
            case FAILED:
            case UNREACHABLE:
            case RUNNING:
            case STOPPING:
                state = STOPPED;
                disconnect();
                break;
        }
        hasDiscovery = true;
    }


    /**
     * Check the MBean server connection and possibly change state in UNREACHABLE
     * or FAILED.
     * @param readall read all monitoring info if server RUNNING
     */
    public void checkit(boolean readall) {
        logger.log(BasicLevel.DEBUG, name);

        switch (state) {
            case RUNNING:
                if (!checkConnection()) {
                    state = UNREACHABLE;
                    disconnect();
                    nbcheck = 0;
                }
                break;
            case FAILED:
                if (checkConnection()) {
                    state = RUNNING;
                }
                break;
            case INITIAL:
                if (checkConnection()) {
                    state = RUNNING;
                } else {
                    state = UNREACHABLE;
                }
                break;
            case UNREACHABLE:
                if (checkConnection()) {
                    state = RUNNING;
                } else {
                    if (hasDiscovery) {
                        // Increment and test nbcheck only if server with
                        // discovery
                        if (nbcheck++ >= maxcheck) {
                            state = FAILED;
                        }
                    }
                }
                break;
            case STOPPING:
                // let stopit set the state
                break;
            case STARTING:
                if (checkConnection()) {
                    state = RUNNING;
                } else {
                    if (nbcheck++ >= maxcheck) {
                        state = UNREACHABLE;
                    }
                }
                break;
            case STOPPED:
                break;
        }
        if (readall && state == RUNNING) {
            // Update some information about the server
            getMonitoringInfo();
        }
    }

    public String getJ2eeObjectName() {
        if (j2eeObjectName == null) {
            j2eeObjectName = J2eeObjectName.J2EEServer(getDomain(), name).toString();
        }
        return j2eeObjectName;
    }

    public String getServerName() {
        return name;
    }

    public ObjectName getServerOn() {
        return serverOn;
    }

    public void setServerOn(ObjectName serverOn) {
        this.serverOn = serverOn;
    }

    public String getClusterDaemonName() {
        if (clusterdaemon == null) {
            return null;
        }
        return clusterdaemon.getName();
    }

    /**
     * Start the Server
     * discovery notification concerning the represented server
     */
    public void startit() {
        logger.log(BasicLevel.DEBUG, name);
        if (clusterdaemon == null) {
            logger.log(BasicLevel.ERROR, "No ClusterDaemon associated to " + name);
            return;
        }
        if (state != RUNNING) {
            state = STARTING;
            clusterdaemon.startServer(name, null);
            nbcheck = 0;
        }
        checkit(true);
    }

    /**
     * Stop the server via the ClusterDaemon
     */
    public void stopit() {
        logger.log(BasicLevel.DEBUG, name);
        if (clusterdaemon == null) {
            logger.log(BasicLevel.ERROR, "No ClusterDaemon associated to " + name);
            return;
        }
        if (state != STOPPED) {
            state = STOPPING;
            if (clusterdaemon.stopServer(name, null)) {
                state = STOPPED;
            }
        }
    }

    /**
     * Deploy a filename on the Remote Server
     * @param fileName Name of the file containing module (or app) to deploy
     * @return operation state
     */
    public String deployModule(String fileName) {

        logger.log(BasicLevel.DEBUG, fileName);

        // Build a DeployAction and register it in DomainMonitor
        DeployAction act = new DeployAction(this, fileName, DeployAction.DEPLOY);
        if (dm == null) {
            logger.log(BasicLevel.ERROR, "Operation reserved to the master");
            return act.getStateAsString();
        }
        if (!dm.registerDeployAction(act)) {
            logger.log(BasicLevel.WARN, "Operation already running for " + fileName);
            return act.getStateAsString();
        }

        // Check connection
        if (!checkConnection()) {
            logger.log(BasicLevel.WARN, "Cannot deploy: Server " + name + " not running");
            act.setError("server not running");
            return act.getStateAsString();
        }

        // operation depends on the type of the file
        String opName = null;
        if (fileName.endsWith(".jar")) {
            opName = "deployJar";
        } else if (fileName.endsWith(".war")) {
            opName = "deployWar";
        } else if (fileName.endsWith(".rar")) {
            opName = "deployRar";
        } else if (fileName.endsWith(".ear")) {
            opName = "deployEar";
        } else {
            logger.log(BasicLevel.ERROR, "Cannot deploy this type of file: " + fileName);
            act.setError("Bad file type : " + fileName);
            return act.getStateAsString();
        }

        String[] params = {fileName };
        String[] signature = {"java.lang.String" };
        try {
            connection.invoke(serverOn, opName, params, signature);
        } catch (InstanceNotFoundException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            act.setError("Remote J2EEServer not registered");
            return act.getStateAsString();
        } catch (MBeanException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            Exception t = e.getTargetException();
            String errmess = t.getMessage();
            Throwable c = t.getCause();
            if (c != null) {
                errmess += " - " + c.getMessage();
            }
            act.setError(errmess);
            return act.getStateAsString();
        } catch (ReflectionException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            act.setError(e.getCause().getMessage());
            return act.getStateAsString();
        } catch (IOException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            act.setError(e.getMessage());
            return act.getStateAsString();
        } catch (RuntimeOperationsException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            Exception t = e.getTargetException();
            String errmess = t.getMessage();
            Throwable c = t.getCause();
            if (c != null) {
                errmess += " - " + c.getMessage();
            }
            act.setError(errmess);
            return act.getStateAsString();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "Unexpected exception: " + e);
            act.setError(e.getMessage());
            return act.getStateAsString();
        }
        act.setOK();
        return act.getStateAsString();
    }

    /**
     * Undeploy a module on the remote server
     * @param fileName Name of the file containing module (or app) to undeploy
     * @return operation state
     */
    public String undeployModule(String fileName) {

        logger.log(BasicLevel.DEBUG, fileName);

        // Build a DeployAction and register it in DomainMonitor
        DeployAction act = new DeployAction(this, fileName, DeployAction.UNDEPLOY);
        if (dm == null) {
            logger.log(BasicLevel.ERROR, "Operation reserved to the master");
            return act.getStateAsString();
        }
        if (!dm.registerDeployAction(act)) {
            logger.log(BasicLevel.WARN, "Operation already running for " + fileName);
            return act.getStateAsString();
        }

        // Check connection
        if (!checkConnection()) {
            logger.log(BasicLevel.WARN, "Cannot undeploy: Server " + name + " not running");
            act.setError("server not running");
            return act.getStateAsString();
        }

        // operation depends on the type of the file
        String opName = null;
        if (fileName.endsWith(".jar")) {
            opName = "unDeployJar";
        } else if (fileName.endsWith(".war")) {
            opName = "unDeployWar";
        } else if (fileName.endsWith(".rar")) {
            opName = "unDeployRar";
        } else if (fileName.endsWith(".ear")) {
            opName = "unDeployEar";
        } else {
            logger.log(BasicLevel.ERROR, "Cannot undeploy this type of file: " + fileName);
            act.setError("Bad file type : " + fileName);
            return act.getStateAsString();
        }

        String[] params = {fileName };
        String[] signature = {"java.lang.String" };
        try {
            connection.invoke(serverOn, opName, params, signature);
        } catch (InstanceNotFoundException e) {
            logger.log(BasicLevel.ERROR, "Cannot undeploy file: " + fileName + " :", e);
            act.setError("Remote J2EEServer not registered");
            return act.getStateAsString();
        } catch (MBeanException e) {
            logger.log(BasicLevel.ERROR, "Cannot undeploy file: " + fileName + " :", e);
            Exception t = e.getTargetException();
            String errmess = t.getMessage();
            Throwable c = t.getCause();
            if (c != null) {
                errmess += " - " + c.getMessage();
            }
            act.setError(errmess);
            return act.getStateAsString();
        } catch (ReflectionException e) {
            logger.log(BasicLevel.ERROR, "Cannot undeploy file: " + fileName + " :", e);
            act.setError(e.getCause().getMessage());
            return act.getStateAsString();
        } catch (IOException e) {
            logger.log(BasicLevel.ERROR, "Cannot undeploy file: " + fileName + " :", e);
            act.setError(e.getMessage());
            return act.getStateAsString();
        }
        act.setOK();
        return act.getStateAsString();
    }

    private static final int BUFFER_SIZE = 1024;

    /**
     * Upload a file on remote server for deployment
     * @param fileName file to upload. One among .war,.jar,.ear,.rar
     * @param replaceExisting
     * @throws IOException
     */
    public void uploadFile(String fileName, boolean replaceExisting) {

        logger.log(BasicLevel.DEBUG, fileName);

        // Build a DeployAction and register it in DomainMonitor
        DeployAction act = new DeployAction(this, fileName, DeployAction.UPLOAD);
        if (dm == null) {
            logger.log(BasicLevel.ERROR, "Operation reserved to the master");
            return;
        }
        if (! dm.registerDeployAction(act)) {
            logger.log(BasicLevel.WARN, "Operation already running for " + fileName);
            return;
        }

        // Check connection
        if (! checkConnection()) {
            logger.log(BasicLevel.WARN, "Cannot upload file: Server " + name + " not running");
            act.setError("server not running");
            return;
        }

        File file = null;
        // first try the file name as it is
        try {
            file = new File(fileName).getCanonicalFile();
        } catch (IOException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file " + fileName, e);
            act.setError(e.getMessage());
            return;
        }
        String deployName = file.getName();
        if (!file.exists()) {
            // if the file was not found try looking for it under JONAS_BASE
            String dir = getFolderDir(fileName);
            if (dir == null) {
                act.setError("Directory not found");
                return;
            }
            file = new File(dir, fileName);
            deployName = fileName;
        }
        if (file == null || !file.exists()) {
            act.setError("File not found");
            return;
        }

        try {
            FileInputStream inputStream = new FileInputStream(file);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            byte[] buf = new byte[BUFFER_SIZE];
            // Read bytes
            int len;
            while ((len = inputStream.read(buf)) > 0) {
                baos.write(buf, 0, len);
            }
            byte[] bytesOfFile = baos.toByteArray();

            Object[] opParams = new Object[] {bytesOfFile, deployName, Boolean.valueOf(replaceExisting)};
            String[] opSignature = new String[] {"[B", "java.lang.String", "boolean"};
            connection.invoke(serverOn, "sendFile", opParams, opSignature);
        } catch (InstanceNotFoundException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file: " + fileName + " :", e);
            act.setError("Remote J2EEServer not registered");
            return;
        } catch (MBeanException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file: " + fileName + " :", e);
            Exception t = e.getTargetException();
            String errmess = t.getMessage();
            Throwable c = t.getCause();
            if (c != null) {
                errmess += " - " + c.getMessage();
            }
            act.setError(errmess);
            return;
        } catch (ReflectionException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file: " + fileName + " :", e);
            act.setError(e.getCause().getMessage());
            return;
        } catch (IOException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file: " + fileName + " :", e);
            act.setError(e.getMessage());
            return;
        }
        act.setOK();
    }

    /**
     * Upload a file on remote server and deploy it.
     * @param fileName file to upload. One among .war,.jar,.ear,.rar
     * @param replaceExisting
     * @throws IOException
     */
    public void uploadDeployModule(String fileName, boolean replaceExisting) {

        logger.log(BasicLevel.DEBUG, fileName);

        // This must be considered as 1 single action, so we cannot just
        // call the 2 methods like that :
        //   uploadFile(filename, replaceExisting);
        //   deployModule(filename);

        // Build a DeployAction and register it in DomainMonitor
        DeployAction act = new DeployAction(this, fileName, DeployAction.UPLOADDEPLOY);
        if (dm == null) {
            logger.log(BasicLevel.ERROR, "Operation reserved to the master");
            return;
        }
        if (! dm.registerDeployAction(act)) {
            logger.log(BasicLevel.WARN, "Operation already running for " + fileName);
            return;
        }

        // Check connection
        if (! checkConnection()) {
            logger.log(BasicLevel.WARN, "Cannot upload file: Server " + name + " not running");
            act.setError("server not running");
            return;
        }

        File file = null;
        // first try the file name as it is
        try {
            file = new File(fileName).getCanonicalFile();
        } catch (IOException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file " + fileName, e);
            act.setError(e.getMessage());
            return;
        }
        String deployName = file.getName();
        if (!file.exists()) {
            // if the file was not found try looking for it under JONAS_BASE
            String dir = getFolderDir(fileName);
            if (dir == null) {
                act.setError("Directory not found");
                return;
            }
            file = new File(dir, fileName);
            deployName = fileName;
        }
        if (file == null || !file.exists()) {
            act.setError("File not found");
            return;
        }

        try {
            FileInputStream inputStream = new FileInputStream(file);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            byte[] buf = new byte[BUFFER_SIZE];
            // Read bytes
            int len;
            while ((len = inputStream.read(buf)) > 0) {
                baos.write(buf, 0, len);
            }
            byte[] bytesOfFile = baos.toByteArray();

            Object[] opParams = new Object[] {bytesOfFile, deployName, Boolean.valueOf(replaceExisting)};
            String[] opSignature = new String[] {"[B", "java.lang.String", "boolean"};
            connection.invoke(serverOn, "sendFile", opParams, opSignature);
        } catch (InstanceNotFoundException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file: " + fileName + " :", e);
            act.setError("Remote J2EEServer not registered");
            return;
        } catch (MBeanException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file: " + fileName + " :", e);
            Exception t = e.getTargetException();
            String errmess = t.getMessage();
            Throwable c = t.getCause();
            if (c != null) {
                errmess += " - " + c.getMessage();
            }
            act.setError(errmess);
            return;
        } catch (ReflectionException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file: " + fileName + " :", e);
            act.setError(e.getCause().getMessage());
            return;
        } catch (IOException e) {
            logger.log(BasicLevel.ERROR, "Cannot upload file: " + fileName + " :", e);
            act.setError(e.getMessage());
            return;
        }
        // Set intermediate state: DEPLOYING
        act.setDeploying();

        // operation depends on the type of the file
        String opName = null;
        if (fileName.endsWith(".jar")) {
            opName = "deployJar";
        } else if (fileName.endsWith(".war")) {
            opName = "deployWar";
        } else if (fileName.endsWith(".rar")) {
            opName = "deployRar";
        } else if (fileName.endsWith(".ear")) {
            opName = "deployEar";
        } else {
            logger.log(BasicLevel.ERROR, "Cannot deploy this type of file: " + fileName);
            return;
        }

        String[] params = {fileName };
        String[] signature = {"java.lang.String" };
        try {
            connection.invoke(serverOn, opName, params, signature);
        } catch (InstanceNotFoundException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            act.setError("Remote J2EEServer not registered");
            return;
        } catch (MBeanException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            Exception t = e.getTargetException();
            String errmess = t.getMessage();
            Throwable c = t.getCause();
            if (c != null) {
                errmess += " - " + c.getMessage();
            }
            act.setError(errmess);
            return;
        } catch (ReflectionException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            act.setError(e.getCause().getMessage());
            return;
        } catch (IOException e) {
            logger.log(BasicLevel.ERROR, "Cannot deploy file: " + fileName + " :", e);
            act.setError(e.getMessage());
            return;
        }
        act.setOK();
    }

    /**
     * Returns the directory that a file would be found in.
     *
     * @param fileName the file to find the directory for.
     * @return The directory of the given file.
     * @throws Exception if an exception is thrown during the deployment
     */
    private String getFolderDir(String fileName) {
        String jBase = JProp.getJonasBase();
        // based on extension
        String dir = null;
        // EJB
        if (fileName.toLowerCase().endsWith(".jar")) {
            dir = jBase + File.separator + "ejbjars";
        } else if (fileName.toLowerCase().endsWith(".war")) {
            // War file
            dir = jBase + File.separator + "webapps";
        } else if (fileName.toLowerCase().endsWith(".ear")) {
            // ear file
            dir = jBase + File.separator + "apps";
        } else if (fileName.toLowerCase().endsWith(".rar")) {
            // rar file
            dir = jBase + File.separator + "rars";
        } else {
            // invalid type
            logger.log(BasicLevel.ERROR, "Invalid extension for " + fileName);
        }
        return dir;
    }

    /**
     * Set information that charaterizes the managed server. First, set infos that does not change,
     * than call getMonitoringInfo() to set info that change in time
     */
    private void setInfo() {
        logger.log(BasicLevel.DEBUG, name);
        if (! checkConnection()) {
            logger.log(BasicLevel.INFO, "Cannot set info for " + name);
            logger.log(BasicLevel.INFO, "Connection is not set already");
            return;
        }
        String domain = getDomain();
        // <domain>:j2eeType=JVM,name=<server>,J2EEServer=<server>
        jvmOn = J2eeObjectName.JVM(domain, name, name);
        // <domain>:j2eeType=JTAResource,name=JTAResource,J2EEServer=<server>
        jtaOn = J2eeObjectName.JTAResource(domain, name, null);
        // <domain>:type=workmanager
        wpoolOn = JonasObjectName.workManager();
        // <domain>:j2eeType=JCAConnectionFactory,name=*,JCAResource=*,J2EEServer=<server>

        String attr = "";
        try {
            if (serverOn != null && connection.isRegistered(serverOn)) {
                attr = "serverVersion";
                JOnASVersion = (String) connection.getAttribute(serverOn, attr);
            }
            if (jvmOn != null && connection.isRegistered(jvmOn)) {
                attr = "javaVersion";
                javaVersion = (String) connection.getAttribute(jvmOn, attr);
                attr = "javaVendor";
                javaVendor = (String) connection.getAttribute(jvmOn, attr);
                attr = "node";
                hostName = (String) connection.getAttribute(jvmOn, attr);
            }
        } catch (Exception e) {
            logger.log(BasicLevel.WARN, "Error while getting attribute " + attr);
            logger.log(BasicLevel.WARN, "serverObjectName=" + serverOn);
            logger.log(BasicLevel.WARN, "jvmObjectName=" + jvmOn);
            throw new ManagementException("Error while getting attribute " + attr + " from " + name, e);
        }
    }

    /**
     * Update non static information for this node.
     */
    public void getMonitoringInfo() {
        logger.log(BasicLevel.DEBUG, name);
        if (jvmOn == null) {
            // setInfo must be called first
            setInfo();
        }
        String attr = "";
        try {
            if (jvmOn != null && connection.isRegistered(jvmOn)) {
                // JVM
                attr = "allThreadsCount";
                allThreadsCount = ((Integer) connection.getAttribute(jvmOn, attr)).intValue();
            }
            // JDK5
            attr = "loadCPU";
            // TODO

            // SERVER
            if (serverOn != null && connection.isRegistered(serverOn)) {
                attr = "currentUsedMemory";
                currentUsedMemory = (Long) connection.getAttribute(serverOn, attr);
                // currentTotalMemory
                attr = "currentTotalMemory";
                currentTotalMemory = (Long) connection.getAttribute(serverOn, attr);
                // protocols
                attr = "protocols";
                protocols = (String) connection.getAttribute(serverOn, attr);
            }
            // TOMCAT
            ObjectName ons = CatalinaObjectName.catalinaThreadPools(getDomain());

            for (Iterator it = connection.queryNames(ons, null).iterator(); it.hasNext();) {
                tomcat = true;
                ObjectName on = (ObjectName) it.next();
                String[] attributes = new String[] {"maxThreads", "currentThreadCount",
                "currentThreadsBusy"};
                AttributeList al = (AttributeList) connection.getAttributes(on, attributes);
                for (int i = 0; i < al.size(); i++) {
                    Attribute at = (Attribute) al.get(i);
                    String name = at.getName();
                    if ("maxThreads".equals(name)) {
                        maxThreadsByConnectorTomcat  = maxThreadsByConnectorTomcat + ((Integer) at.getValue()).intValue();
                    } else if ("currentThreadCount".equals(name)) {
                        currentThreadCountByConnectorTomcat = currentThreadCountByConnectorTomcat + ((Integer) at.getValue()).intValue();
                    } else if ("currentThreadsBusy".equals(name)) {
                        currentThreadBusyByConnectorTomcat = currentThreadBusyByConnectorTomcat + ((Integer) at.getValue()).intValue();
                    }
                }
            }
            ons = CatalinaObjectName.catalinaGlobalRequestProcessors(getDomain());
            for (Iterator it = connection.queryNames(ons, null).iterator(); it.hasNext();) {
                tomcat = true;
                ObjectName on = (ObjectName) it.next();
                String[] attributes = new String[] {
                        "bytesReceived", "bytesSent", "errorCount", "maxTime",
                        "processingTime", "requestCount"
                };
                AttributeList al = (AttributeList) connection.getAttributes(on, attributes);
                for (int i = 0; i < al.size(); i++) {
                    Attribute at = (Attribute) al.get(i);
                    String name = at.getName();
                    if ("bytesReceived".equals(name)) {
                        bytesReceivedByConnectorTomcat += ((Long) at.getValue()).longValue();
                    } else if ("bytesSent".equals(name)) {
                        bytesSentByConnectorTomcat += ((Long) at.getValue()).longValue();
                    } else if ("errorCount".equals(name)) {
                        errorCountByConnectorTomcat += ((Integer) at.getValue()).intValue();
                    } else if ("processingTime".equals(name)) {
                        processingTimeByConnectorTomcat += ((Long) at.getValue()).longValue();
                    } else if ("requestCount".equals(name)) {
                        requestCountByConnectorTomcat += ((Integer) at.getValue()).intValue();
                    }
                }
            }

            // JTA
            if (jtaOn != null && connection.isRegistered(jtaOn)) {
                attr = "totalBegunTransactions";
                totalBegunTransactions = ((Integer) connection.getAttribute(jtaOn, attr)).intValue();
                attr = "totalCommittedTransactions";
                totalCommittedTransactions = ((Integer) connection.getAttribute(jtaOn, attr)).intValue();
                attr = "totalCurrentTransactions";
                totalCurrentTransactions = ((Integer) connection.getAttribute(jtaOn, attr)).intValue();
                attr = "totalExpiredTransactions";
                totalExpiredTransactions = ((Integer) connection.getAttribute(jtaOn, attr)).intValue();
                attr = "totalRolledbackTransactions";
                totalRolledbackTransactions = ((Integer) connection.getAttribute(jtaOn, attr)).intValue();
            }

            // WORKER POOL
            if (wpoolOn != null && connection.isRegistered(wpoolOn)) {
                attr = "currentPoolSize";
                currentWorkerPoolSize = ((Integer) connection.getAttribute(wpoolOn, attr)).intValue();
                attr = "maxPoolSize";
                maxWorkerPoolSize = ((Integer) connection.getAttribute(wpoolOn, attr)).intValue();
                attr = "minPoolSize";
                minWorkerPoolSize = ((Integer) connection.getAttribute(wpoolOn, attr)).intValue();
            }

            // TODO rethink this list. Only some of them are interesting.
            attr = "busyMaxJCAConnection";
            attr = "busyMinJCAConnection";
            attr = "connectionFailuresJCAConnection";
            attr = "connectionLeaksJCAConnection";
            attr = "currentBusyJCAConnection";
            attr = "currentInTxJCAConnection";
            attr = "currentOpenedJCAConnection";
            attr = "currentWaitersJCAConnection";
            attr = "initPoolSizeJCAConnection";
            attr = "poolMaxSizeJCAConnection";
            attr = "poolMinSizeJCAConnection";
            attr = "poolMaxWaitersJCAConnection";
            attr = "poolMaxWaitTimeJCAConnection";
            attr = "poolMaxOpentimeJCAConnection";
            attr = "physicalConnectionOpenedCountJCAConnection";
            attr = "pstmtMaxJCAConnection";
            attr = "rejectedFullJCAConnection";
            attr = "rejectedOpenJCAConnection";
            attr = "rejectedOtherJCAConnection";
            attr = "rejectedTimeoutJCAConnection";
            attr = "xaServedOpenJCAConnection";
            attr = "waiterCountJCAConnection";
            attr = "waitersHighJCAConnection";
            attr = "waitersHighRecentJCAConnection";
            attr = "waitingHighJCAConnection";
            attr = "waitingHighRecentJCAConnection";
            attr = "waitingTimeJCAConnection";
            // TODO

            // JMS
            attr = "jmsQueuesNbMsgsReceiveSinceCreation";
            attr = "jmsQueuesNbMsgsSendToDMQSinceCreation";
            attr = "jmsQueuesNbMsgsDeliverSinceCreation";
            attr = "jmsTopicsNbMsgsReceiveSinceCreation";
            attr = "jmsTopicsNbMsgsSendToDMQSinceCreation";
            attr = "jmsTopicsNbMsgsDeliverSinceCreation";
            // TODO

            // EJB
            attr = "currentNumberOfBMP";
            attr = "currentNumberOfCMP";
            attr = "currentNumberOfEJB";
            attr = "currentNumberOfMDB";
            attr = "currentNumberOfSBF";
            attr = "currentNumberOfSBL";
            attr = "currentNumberOfEBInMemory";
            attr = "maxNumberOfEBInMemory";
            attr = "minPoolSizeEB";
            attr = "poolSizeEB";
            attr = "currentNumberOfSBFInMemory";
            attr = "maxNumberOfSBFInMemory";
            attr = "minPoolSizeSBF";
            attr = "poolSizeSBF";
            attr = "currentNumberOfSBLInMemory";
            attr = "maxNumberOfSBLInMemory";
            attr = "minPoolSizeSBL";
            attr = "poolSizeSBL";
            // TODO


        } catch (Exception e) {
            logger.log(BasicLevel.WARN, "Error while getting attribute " + attr);
        }
    }

    public int getMaxcheck() {
        return maxcheck;
    }

    public void setMaxcheck(int maxcheck) {
        this.maxcheck = maxcheck;
    }

    public int getAllThreadsCount() {
        return allThreadsCount;
    }

    public int getBusyMaxJCAConnection() {
        return busyMaxJCAConnection;
    }

    public int getBusyMinJCAConnection() {
        return busyMinJCAConnection;
    }

    public long getBytesReceivedByConnectorTomcat() {
        return bytesReceivedByConnectorTomcat;
    }

    public long getBytesSentByConnectorTomcat() {
        return bytesSentByConnectorTomcat;
    }

    public int getConnectionFailuresJCAConnection() {
        return connectionFailuresJCAConnection;
    }

    public int getConnectionLeaksJCAConnection() {
        return connectionLeaksJCAConnection;
    }

    public int getCurrentBusyJCAConnection() {
        return currentBusyJCAConnection;
    }

    public int getCurrentInTxJCAConnection() {
        return currentInTxJCAConnection;
    }

    public int getCurrentNumberOfBMP() {
        return currentNumberOfBMP;
    }

    public int getCurrentNumberOfCMP() {
        return currentNumberOfCMP;
    }

    public int getCurrentNumberOfEBInMemory() {
        return currentNumberOfEBInMemory;
    }

    public int getCurrentNumberOfEJB() {
        return currentNumberOfEJB;
    }

    public int getCurrentNumberOfMDB() {
        return currentNumberOfMDB;
    }

    public int getCurrentNumberOfSBF() {
        return currentNumberOfSBF;
    }

    public int getCurrentNumberOfSBFInMemory() {
        return currentNumberOfSBFInMemory;
    }

    public int getCurrentNumberOfSBL() {
        return currentNumberOfSBL;
    }

    public int getCurrentNumberOfSBLInMemory() {
        return currentNumberOfSBLInMemory;
    }

    public int getCurrentOpenedJCAConnection() {
        return currentOpenedJCAConnection;
    }

    public int getCurrentThreadBusyByConnectorTomcat() {
        return currentThreadBusyByConnectorTomcat;
    }

    public int getCurrentThreadCountByConnectorTomcat() {
        return currentThreadCountByConnectorTomcat;
    }

    public Long getCurrentTotalMemory() {
        return currentTotalMemory;
    }

    public Long getCurrentUsedMemory() {
        return currentUsedMemory;
    }

    public int getCurrentWaitersJCAConnection() {
        return currentWaitersJCAConnection;
    }

    public int getCurrentWorkerPoolSize() {
        return currentWorkerPoolSize;
    }

    public int getErrorCountByConnectorTomcat() {
        return errorCountByConnectorTomcat;
    }

    public String getHostName() {
        return hostName;
    }

    public int getInitPoolSizeJCAConnection() {
        return initPoolSizeJCAConnection;
    }

    public String getJavaVendor() {
        return javaVendor;
    }

    public String getJavaVersion() {
        return javaVersion;
    }

    public int getJmsQueuesNbMsgsDeliverSinceCreation() {
        return jmsQueuesNbMsgsDeliverSinceCreation;
    }

    public int getJmsQueuesNbMsgsReceiveSinceCreation() {
        return jmsQueuesNbMsgsReceiveSinceCreation;
    }

    public int getJmsQueuesNbMsgsSendToDMQSinceCreation() {
        return jmsQueuesNbMsgsSendToDMQSinceCreation;
    }

    public int getJmsTopicsNbMsgsDeliverSinceCreation() {
        return jmsTopicsNbMsgsDeliverSinceCreation;
    }

    public int getJmsTopicsNbMsgsReceiveSinceCreation() {
        return jmsTopicsNbMsgsReceiveSinceCreation;
    }

    public int getJmsTopicsNbMsgsSendToDMQSinceCreation() {
        return jmsTopicsNbMsgsSendToDMQSinceCreation;
    }

    public String getJOnASVersion() {
        return JOnASVersion;
    }

    public String getLoadCPU() {
        return loadCPU;
    }

    public int getMaxNumberOfEBInMemory() {
        return maxNumberOfEBInMemory;
    }

    public int getMaxNumberOfSBFInMemory() {
        return maxNumberOfSBFInMemory;
    }

    public int getMaxNumberOfSBLInMemory() {
        return maxNumberOfSBLInMemory;
    }

    public int getMaxThreadsByConnectorTomcat() {
        return maxThreadsByConnectorTomcat;
    }

    public int getMaxWorkerPoolSize() {
        return maxWorkerPoolSize;
    }

    public MBeanServer getMbeanServer() {
        return mbeanServer;
    }

    public int getMinPoolSizeEB() {
        return minPoolSizeEB;
    }

    public int getMinPoolSizeSBF() {
        return minPoolSizeSBF;
    }

    public int getMinPoolSizeSBL() {
        return minPoolSizeSBL;
    }

    public int getMinWorkerPoolSize() {
        return minWorkerPoolSize;
    }

    public int getPhysicalConnectionOpenedCountJCAConnection() {
        return physicalConnectionOpenedCountJCAConnection;
    }

    public int getPoolMaxOpentimeJCAConnection() {
        return poolMaxOpentimeJCAConnection;
    }

    public int getPoolMaxSizeJCAConnection() {
        return poolMaxSizeJCAConnection;
    }

    public int getPoolMaxWaitersJCAConnection() {
        return poolMaxWaitersJCAConnection;
    }

    public int getPoolMaxWaitTimeJCAConnection() {
        return poolMaxWaitTimeJCAConnection;
    }

    public int getPoolMinSizeJCAConnection() {
        return poolMinSizeJCAConnection;
    }

    public int getPoolSizeEB() {
        return poolSizeEB;
    }

    public int getPoolSizeSBF() {
        return poolSizeSBF;
    }

    public int getPoolSizeSBL() {
        return poolSizeSBL;
    }

    public long getProcessingTimeByConnectorTomcat() {
        return processingTimeByConnectorTomcat;
    }

    public String getProtocols() {
        return protocols;
    }

    public int getPstmtMaxJCAConnection() {
        return pstmtMaxJCAConnection;
    }

    public int getRejectedFullJCAConnection() {
        return rejectedFullJCAConnection;
    }

    public int getRejectedOpenJCAConnection() {
        return rejectedOpenJCAConnection;
    }

    public int getRejectedOtherJCAConnection() {
        return rejectedOtherJCAConnection;
    }

    public int getRejectedTimeoutJCAConnection() {
        return rejectedTimeoutJCAConnection;
    }

    public int getRequestCountByConnectorTomcat() {
        return requestCountByConnectorTomcat;
    }

    public int getTotalBegunTransactions() {
        return totalBegunTransactions;
    }

    public int getTotalCommittedTransactions() {
        return totalCommittedTransactions;
    }

    public int getTotalCurrentTransactions() {
        return totalCurrentTransactions;
    }

    public int getTotalExpiredTransactions() {
        return totalExpiredTransactions;
    }

    public int getTotalRolledbackTransactions() {
        return totalRolledbackTransactions;
    }

    public int getWaiterCountJCAConnection() {
        return waiterCountJCAConnection;
    }

    public int getWaitersHighJCAConnection() {
        return waitersHighJCAConnection;
    }

    public int getWaitersHighRecentJCAConnection() {
        return waitersHighRecentJCAConnection;
    }

    public int getWaitingHighJCAConnection() {
        return waitingHighJCAConnection;
    }

    public int getWaitingHighRecentJCAConnection() {
        return waitingHighRecentJCAConnection;
    }

    public int getWaitingTimeJCAConnection() {
        return waitingTimeJCAConnection;
    }

    public int getXaServedOpenJCAConnection() {
        return xaServedOpenJCAConnection;
    }


    public boolean isTomcat() {
        return tomcat;
    }

}